@@ -1,8 +1,12 @@
+from ._version import get_versions
+from .aggregate_purepy import aggregate as aggregate_py
+
+
 def dummy_no_impl(*args, **kwargs):
     raise NotImplementedError("You may need to install another package (numpy, "
                               "weave, or numba) to access a working implementation.")
 
-from .aggregate_purepy import aggregate as aggregate_py
+
 aggregate = aggregate_py
 
 try:
@@ -14,11 +18,8 @@ else:
     from .aggregate_numpy import aggregate
     aggregate_np = aggregate
     from .aggregate_numpy_ufunc import aggregate as aggregate_ufunc
-    from .utils_numpy import (multi_arange,
-                              label_contiguous_1d,
-                              relabel_groups_masked,
-                              relabel_groups_unique,
-                              unpack)
+    from .utils_numpy import (label_contiguous_1d, multi_arange, relabel_groups_masked,
+                              relabel_groups_unique, unpack)
 
 
 try:
@@ -29,7 +30,7 @@ try:
 except ImportError:
     aggregate_wv = None
 else:
-    from .aggregate_weave import aggregate as aggregate_wv, step_indices, step_count
+    from .aggregate_weave import aggregate as aggregate_wv, step_count, step_indices
     aggregate = aggregate_wv
 
 
@@ -38,13 +39,13 @@ try:
 except ImportError:
     aggregate_nb = None
 else:
-    from .aggregate_numba import aggregate as aggregate_nb, step_indices, step_count
+    from .aggregate_numba import aggregate as aggregate_nb, step_count, step_indices
     aggregate = aggregate_nb
 
 
 def uaggregate(group_idx, a, **kwargs):
     return unpack(group_idx, aggregate(group_idx, a, **kwargs))
 
-from ._version import get_versions
+
 __version__ = get_versions()['version']
 del get_versions
