@@ -1,12 +1,10 @@
 from __future__ import division
 
-import math
 import itertools
+import math
 import operator
 
-from .utils import (get_func, aliasing, funcs_no_separate_nan,
-                    aggregate_common_doc, isstr)
-
+from .utils import aggregate_common_doc, aliasing, funcs_no_separate_nan, get_func, isstr
 
 # min - builtin
 # max - builtin
@@ -57,11 +55,15 @@ def _allnan(x):
 
 def _argmax(x_and_idx):
     return max(x_and_idx, key=operator.itemgetter(1))[0]
+
+
 _argmax.x_and_idx = True  # tell aggregate what to use as first arg
 
 
 def _argmin(x_and_idx):
     return min(x_and_idx, key=operator.itemgetter(1))[0]
+
+
 _argmin.x_and_idx = True  # tell aggregate what to use as first arg
 
 
@@ -90,7 +92,11 @@ def aggregate(group_idx, a, func='sum', size=None, fill_value=0, order=None,
 
     # Check for 2d group_idx
     if size is None:
-        size = 1 + int(max(group_idx))
+        try:
+            size = 1 + int(max(group_idx))
+        except (TypeError, ValueError):
+            raise NotImplementedError("pure python implementation doesn't"
+                                      " accept ndim idx input.")
 
     for i in group_idx:
         try:
