@@ -26,30 +26,40 @@ def _impl_name(impl):
 
 
 _not_implemented_by_impl_name = {
-    'numpy': ['cumprod','cummax', 'cummin'],
-    'purepy': ['cumprod','cummax', 'cummin'],
-    'numba': ('array', 'list'),
-    'pandas': ('array', 'list'),
-    'weave':  ('argmin', 'argmax', 'array', 'list', 'cumsum',
-               '<lambda>', 'func_preserve_order', 'func_arbitrary')}
+    'numpy':  ('cumprod','cummax', 'cummin'),
+    'purepy': ('cumsum', 'cumprod','cummax', 'cummin', 'sumofsquares'),
+    'numba':  ('array', 'list', 'sort'),
+    'pandas': ('array', 'list', 'sort', 'sumofsquares', 'nansumofsquares'),
+    'weave':  ('argmin', 'argmax', 'array', 'list', 'sort', 'cumsum', 'cummax', 'cummin',
+               'nanargmin', 'nanargmax', 'sumofsquares', 'nansumofsquares',
+               '<lambda>', 'custom_callable'),
+    'ufunc':  'NO_CHECK'}
+
 
 def _wrap_notimplemented_xfail(impl, name=None):
     """Some implementations lack some functionality. That's ok, let's xfail that instead of raising errors."""
 
-    def _try_xfail(*args, **kwargs):
+    def try_xfail(*args, **kwargs):
         try:
             return impl(*args, **kwargs)
         except NotImplementedError as e:
+            impl_name = impl.__module__.split('_')[-1]
             func = kwargs.pop('func', None)
             if callable(func):
                 func = func.__name__
-            wrap_funcs = _not_implemented_by_impl_name.get(func, None)
-            if wrap_funcs is None or func in wrap_funcs:
+            not_implemented_ok = _not_implemented_by_impl_name.get(impl_name, [])
+            if not_implemented_ok == 'NO_CHECK' or func in not_implemented_ok:
                 pytest.xfail("Functionality not implemented")
             else:
                 raise e
     if name:
-        _try_xfail.__name__ = name
+        try_xfail.__name__ = name
     else:
-        _try_xfail.__name__ = impl.__name__
-    return _try_xfail
+        try_xfail.__name__ = impl.__name__
+    return try_xfail
+
+
+func_list = ('sum', 'prod', 'min', 'max', 'all', 'any', 'mean', 'std', 'var', 'len',
+             'argmin', 'argmax', 'anynan', 'allnan', 'cumsum', 'sumofsquares',
+             'nansum', 'nanprod', 'nanmin', 'nanmax', 'nanmean', 'nanstd', 'nanvar','nanlen',
+             'nanargmin', 'nanargmax', 'nansumofsquares')
