@@ -3,12 +3,13 @@ import numpy as np
 
 from . import aggregate_weave, aggregate_numba, _impl_name
 _implementations = [aggregate_weave, aggregate_numba]
+_implementations = [i for i in _implementations if i is not None]
 
 
 @pytest.fixture(params=_implementations, ids=_impl_name)
 def aggregate_nb_wv(request):
     if request.param is None:
-        pytest.xfail("Implementation not available")
+        pytest.skip("Implementation not available")
     return request.param
 
 
