@@ -1,7 +1,7 @@
 """Common helper functions for typing and general numpy tools."""
 import numpy as np
 
-from .utils import get_aliasing, check_boolean
+from .utils import check_boolean, get_aliasing
 
 _alias_numpy = {
     np.add: 'sum',
@@ -92,23 +92,24 @@ def minimum_dtype(x, dtype=np.bool_):
 
 def minimum_dtype_scalar(x, dtype, a):
     if dtype is None:
-        dtype = np.dtype(type(a)) if isinstance(a, (int, float))\
-                                  else a.dtype
+        dtype = np.dtype(type(a)) if isinstance(a, (int, float)) else a.dtype
     return minimum_dtype(x, dtype)
 
 
 _forced_types = {
-    'array': np.object,
-    'all': np.bool_,
-    'any': np.bool_,
-    'nanall': np.bool_,
-    'nanany': np.bool_,
+    'array': object,
+    'all': bool,
+    'any': bool,
+    'nanall': bool,
+    'nanany': bool,
     'len': np.int64,
     'nanlen': np.int64,
-    'allnan': np.bool_,
-    'anynan': np.bool_,
+    'allnan': bool,
+    'anynan': bool,
     'argmax': np.int64,
     'argmin': np.int64,
+    'nanargmin': np.int64,
+    'nanargmax': np.int64,
 }
 _forced_float_types = {'mean', 'var', 'std', 'nanmean', 'nanvar', 'nanstd'}
 _forced_same_type = {'min', 'max', 'first', 'last', 'nanmin', 'nanmax',
@@ -168,6 +169,24 @@ def check_dtype(dtype, func_str, a, n):
                         return a_dtype
 
 
+def minval(fill_value, dtype):
+    dtype = minimum_dtype(fill_value, dtype)
+    if issubclass(dtype.type, np.floating):
+        return -np.inf
+    if issubclass(dtype.type, np.integer):
+        return  np.iinfo(dtype).min
+    return np.finfo(dtype).min
+
+
+def maxval(fill_value, dtype):
+    dtype = minimum_dtype(fill_value, dtype)
+    if issubclass(dtype.type, np.floating):
+        return np.inf
+    if issubclass(dtype.type, np.integer):
+        return  np.iinfo(dtype).max
+    return np.finfo(dtype).max
+
+
 def check_fill_value(fill_value, dtype, func=None):
     if func in ('all', 'any', 'allnan', 'anynan'):
         check_boolean(fill_value)
@@ -189,15 +208,67 @@ def check_group_idx(group_idx, a=None, check_min=True):
         raise ValueError("group_idx contains negative indices")
 
 
+def _ravel_group_idx(group_idx, a, axis, size, order, method="ravel"):
+    ndim_a = a.ndim
+    # Create the broadcast-ready multidimensional indexing.
+    # Note the user could do this themselves, so this is
+    # very much just a convenience.
+    size_in = int(np.max(group_idx)) + 1 if size is None else size
+    group_idx_in = group_idx
+    group_idx = []
+    size = []
+    for ii, s in enumerate(a.shape):
+        if method == "ravel":
+            ii_idx = group_idx_in if ii == axis else np.arange(s)
+            ii_shape = [1] * ndim_a
+            ii_shape[ii] = s
+            group_idx.append(ii_idx.reshape(ii_shape))
+        size.append(size_in if ii == axis else s)
+    # Use the indexing, and return. It's a bit simpler than
+    # using trying to keep all the logic below happy
+    if method == "ravel":
+        group_idx = np.ravel_multi_index(group_idx, size, order=order,
+                                         mode='raise')
+    elif method == "offset":
+        group_idx = offset_labels(group_idx_in, a.shape, axis, order, size_in)
+    return group_idx, size
+
+
+def offset_labels(group_idx, inshape, axis, order, size):
+    """
+    Offset group labels by dimension. This is used when we
+    reduce over a subset of the dimensions of by. It assumes that the reductions
+    dimensions have been flattened in the last dimension
+    Copied from
+    https://stackoverflow.com/questions/46256279/bin-elements-per-row-vectorized-2d-bincount-for-numpy
+    """
+
+    newaxes = tuple(ax for ax in range(len(inshape)) if ax != axis)
+    group_idx = np.broadcast_to(np.expand_dims(group_idx, newaxes), inshape)
+    if axis not in (-1, len(inshape) - 1):
+        group_idx = np.moveaxis(group_idx, axis, -1)
+    newshape = group_idx.shape[:-1] + (-1,)
+
+    group_idx = (group_idx +
+                 np.arange(np.prod(newshape[:-1]), dtype=int).reshape(newshape)
+                 * size
+                 )
+    if axis not in (-1, len(inshape) - 1):
+        return np.moveaxis(group_idx, -1, axis)
+    else:
+        return group_idx
+
+
 def input_validation(group_idx, a, size=None, order='C', axis=None,
-                     ravel_group_idx=True, check_bounds=True):
+                     ravel_group_idx=True, check_bounds=True, method="ravel", func=None):
     """ Do some fairly extensive checking of group_idx and a, trying to
     give the user as much help as possible with what is wrong. Also,
     convert ndim-indexing to 1d indexing.
     """
-    if not isinstance(a, (int, float, complex)):
+    if not isinstance(a, (int, float, complex)) and not is_duck_array(a):
         a = np.asanyarray(a)
-    group_idx = np.asanyarray(group_idx)
+    if not is_duck_array(group_idx):
+        group_idx = np.asanyarray(group_idx)
 
     if not np.issubdtype(group_idx.dtype, np.integer):
         raise TypeError("group_idx must be of integer type")
@@ -230,26 +301,18 @@ def input_validation(group_idx, a, size=None, order='C', axis=None,
             raise NotImplementedError("when using axis arg, size must be"
                                       "None or scalar.")
         else:
-            # Create the broadcast-ready multidimensional indexing.
-            # Note the user could do this themselves, so this is
-            # very much just a convenience.
-            size_in = int(np.max(group_idx)) + 1 if size is None else size
-            group_idx_in = group_idx
-            group_idx = []
-            size = []
-            for ii, s in enumerate(a.shape):
-                ii_idx = group_idx_in if ii == axis else np.arange(s)
-                ii_shape = [1] * ndim_a
-                ii_shape[ii] = s
-                group_idx.append(ii_idx.reshape(ii_shape))
-                size.append(size_in if ii == axis else s)
-            # Use the indexing, and return. It's a bit simpler than
-            # using trying to keep all the logic below happy
-            group_idx = np.ravel_multi_index(group_idx, size, order=order,
-                                             mode='raise')
+            is_form_3 = group_idx.ndim == 1 and a.ndim > 1 and axis is not None
+            orig_shape = a.shape if is_form_3 else group_idx.shape
+            if isinstance(func, str) and "arg" in func:
+                unravel_shape = orig_shape
+            else:
+                unravel_shape = None
+
+            group_idx, size = _ravel_group_idx(group_idx, a, axis, size, order, method=method)
             flat_size = np.prod(size)
             ndim_idx = ndim_a
-            return group_idx.ravel(), a.ravel(), flat_size, ndim_idx, size
+            size = orig_shape if is_form_3 and not callable(func) and "cum" in func else size
+            return group_idx.ravel(), a.ravel(), flat_size, ndim_idx, size, unravel_shape
 
     if ndim_idx == 1:
         if size is None:
@@ -263,7 +326,7 @@ def input_validation(group_idx, a, size=None, order='C', axis=None,
         flat_size = size
     else:
         if size is None:
-            size = int(np.max(group_idx, axis=1)) + 1
+            size = np.max(group_idx, axis=1).astype(int) + 1
         elif np.isscalar(size):
             raise ValueError("output size must be of length %d"
                              % len(group_idx))
@@ -280,14 +343,14 @@ def input_validation(group_idx, a, size=None, order='C', axis=None,
         raise ValueError("group_idx and a must be of the same length, or a"
                          " can be scalar")
 
-    return group_idx, a, flat_size, ndim_idx, size
+    return group_idx, a, flat_size, ndim_idx, size, None
 
 
 ### General tools ###
 
 
 def unpack(group_idx, ret):
-    """ Take an aggregate packed array and uncompress it to the size of group_idx. 
+    """ Take an aggregate packed array and uncompress it to the size of group_idx.
         This is equivalent to ret[group_idx].
     """
     return ret[group_idx]
@@ -311,17 +374,17 @@ def nanlast(x):
 
 def multi_arange(n):
     """By example:
-    
+
         #    0  1  2  3  4  5  6  7  8
         n = [0, 0, 3, 0, 0, 2, 0, 2, 1]
         res = [0, 1, 2, 0, 1, 0, 1, 0]
 
     That is it is equivalent to something like this :
-    
+
         hstack((arange(n_i) for n_i in n))
-        
+
     This version seems quite a bit faster, at least for some
-    possible inputs, and at any rate it encapsulates a task 
+    possible inputs, and at any rate it encapsulates a task
     in a function.
     """
     if n.ndim != 1:
@@ -336,24 +399,24 @@ def multi_arange(n):
 
 
 def label_contiguous_1d(X):
-    """ 
-    WARNING: API for this function is not liable to change!!!    
-    
+    """
+    WARNING: API for this function is not liable to change!!!
+
     By example:
 
         X =      [F T T F F T F F F T T T]
         result = [0 1 1 0 0 2 0 0 0 3 3 3]
-    
+
     Or:
         X =      [0 3 3 0 0 5 5 5 1 1 0 2]
         result = [0 1 1 0 0 2 2 2 3 3 0 4]
-    
+
     The ``0`` or ``False`` elements of ``X`` are labeled as ``0`` in the output. If ``X``
     is a boolean array, each contiguous block of ``True`` is given an integer
     label, if ``X`` is not boolean, then each contiguous block of identical values
     is given an integer label. Integer labels are 1, 2, 3,..... (i.e. start a 1
     and increase by 1 for each block with no skipped numbers.)
-    
+
     """
 
     if X.ndim != 1:
@@ -378,10 +441,10 @@ def label_contiguous_1d(X):
 def relabel_groups_unique(group_idx):
     """
     See also ``relabel_groups_masked``.
-    
+
     keep_group:  [0 3 3 3 0 2 5 2 0 1 1 0 3 5 5]
     ret:         [0 3 3 3 0 2 4 2 0 1 1 0 3 4 4]
-    
+
     Description of above: unique groups in input was ``1,2,3,5``, i.e.
     ``4`` was missing, so group 5 was relabled to be ``4``.
     Relabeling maintains order, just "compressing" the higher numbers
@@ -397,24 +460,24 @@ def relabel_groups_unique(group_idx):
 def relabel_groups_masked(group_idx, keep_group):
     """
     group_idx: [0 3 3 3 0 2 5 2 0 1 1 0 3 5 5]
-   
+
                  0 1 2 3 4 5
     keep_group: [0 1 0 1 1 1]
-    
+
     ret:       [0 2 2 2 0 0 4 0 0 1 1 0 2 4 4]
-    
+
     Description of above in words: remove group 2, and relabel group 3,4, and 5
     to be 2, 3 and 4 respecitvely, in order to fill the gap.  Note that group 4 was never used
     in the input group_idx, but the user supplied mask said to keep group 4, so group
     5 is only moved up by one place to fill the gap created by removing group 2.
-    
+
     That is, the mask describes which groups to remove,
     the remaining groups are relabled to remove the gaps created by the falsy
     elements in ``keep_group``.  Note that ``keep_group[0]`` has no particular meaning because it refers
     to the zero group which cannot be "removed".
 
     ``keep_group`` should be bool and ``group_idx`` int.
-    Values in ``group_idx`` can be any order, and 
+    Values in ``group_idx`` can be any order, and
     """
 
     keep_group = keep_group.astype(bool, copy=not keep_group[0])
@@ -424,3 +487,32 @@ def relabel_groups_masked(group_idx, keep_group):
     relabel = np.zeros(keep_group.size, dtype=group_idx.dtype)
     relabel[keep_group] = np.arange(np.count_nonzero(keep_group))
     return relabel[group_idx]
+
+
+def is_duck_array(value):
+    """
+    This function was copied from xarray/core/utils.py under the terms
+    of Xarray's Apache-2 license
+    """
+    if isinstance(value, np.ndarray):
+        return True
+    return (
+        hasattr(value, "ndim")
+        and hasattr(value, "shape")
+        and hasattr(value, "dtype")
+        and hasattr(value, "__array_function__")
+        and hasattr(value, "__array_ufunc__")
+    )
+
+
+def iscomplexobj(x):
+    """
+    Copied from np.iscomplexobj so that we place fewer requirements
+    on duck array types.
+    """
+    try:
+        dtype = x.dtype
+        type_ = dtype.type
+    except AttributeError:
+        type_ = np.asarray(x).dtype.type
+    return issubclass(type_, np.complexfloating)
